'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = maybeYouMeant;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _fastLevenshtein = require('fast-levenshtein');

var _fastLevenshtein2 = _interopRequireDefault(_fastLevenshtein);

var _arrify = require('arrify');

var _arrify2 = _interopRequireDefault(_arrify);

var _lodash = require('lodash.some');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getDisplayName = function getDisplayName(c) {
  return c.displayName || c.name || 'Component';
};
var noop = function noop() {};
var toRegExp = function toRegExp(s) {
  return typeof s === 'string' ? new RegExp('^' + s + '$') : s;
};
var normalizeOpts = function normalizeOpts() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$include = _ref.include,
      include = _ref$include === undefined ? /./ : _ref$include,
      _ref$exclude = _ref.exclude,
      exclude = _ref$exclude === undefined ? /[^a-zA-Z0-9]/ : _ref$exclude,
      _ref$maxDistance = _ref.maxDistance,
      maxDistance = _ref$maxDistance === undefined ? 3 : _ref$maxDistance;

  return {
    include: (0, _arrify2.default)(include).map(toRegExp),
    exclude: (0, _arrify2.default)(exclude).map(toRegExp),
    maxDistance: maxDistance
  };
};

function shouldPatch(displayName, _ref2) {
  var include = _ref2.include,
      exclude = _ref2.exclude;

  var isIncluded = (0, _lodash2.default)(include, function (r) {
    return r.test(displayName);
  });
  var isExcluded = (0, _lodash2.default)(exclude, function (r) {
    return r.test(displayName);
  });
  return isIncluded && !isExcluded;
}

function checkProps(Component, props, maxDistance) {
  var displayName = getDisplayName(Component);
  var propTypeKeys = Object.keys(Component.propTypes || {});
  var propKeys = Object.keys(props);

  propKeys.forEach(function (prop) {
    if (!~propTypeKeys.indexOf(prop)) {
      propTypeKeys.forEach(function (propType) {
        var dist = _fastLevenshtein2.default.get(prop, propType);
        if (dist <= maxDistance) {
          console.warn(displayName + ': got prop "' + prop + '".' + (' Maybe you meant "' + propType + '"?'));
        }
      });
    }
  });
}

function patch(Component, opts) {
  var maxDistance = opts.maxDistance;

  var displayName = getDisplayName(Component);
  if (Component.__isMaybeYouMeantPatched || !shouldPatch(displayName, opts)) {
    return Component;
  }

  Component.__isMaybeYouMeantPatched = true;

  if (typeof Component.prototype.render !== 'function') {
    var Wrapper = function Wrapper(props) {
      checkProps(Component, props, maxDistance);
      return _react2.default.createElement(Component, props);
    };
    Wrapper.displayName = 'MaybeYouMeant(' + displayName + ')';
    return Wrapper;
  }

  var _componentDidMount = Component.prototype.componentDidMount || noop;
  var _componentDidUpdate = Component.prototype.componentDidUpdate || noop;

  Component.prototype.componentDidMount = function () {
    checkProps(Component, this.props, maxDistance);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _componentDidMount.call.apply(_componentDidMount, [this].concat(args));
  };

  Component.prototype.componentDidUpdate = function () {
    checkProps(Component, this.props, maxDistance);

    for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    return _componentDidUpdate.call.apply(_componentDidUpdate, [this].concat(args));
  };

  return Component;
}

function maybeYouMeant() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var _createElement = _react2.default.createElement;
  opts = normalizeOpts(opts);

  _react2.default.createElement = function (Component) {
    if (typeof Component === 'function') {
      Component = patch(Component, opts);
    }

    for (var _len3 = arguments.length, rest = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
      rest[_key3 - 1] = arguments[_key3];
    }

    return _createElement.call.apply(_createElement, [_react2.default, Component].concat(rest));
  };

  _react2.default.__restoreMaybeYouMeant = function () {
    _react2.default.createElement = _createElement;
  };
}